#include "ti83plus.inc"
#include "macrodie.inc"

.org $4000

; change this to your ID
#define DEVELOPER_ID    $01,$23,$45

EXT_APP = 1                         ; This definition is required of all apps

; This is the application header definition area required for all apps.

 .db $80, $0F                       ; Field: Program length
 .db 0, 0, 0, 0                     ; Length = 0 (N/A for unsigned apps)

#ifdef SIGN_APP
 .db $80, $13                       ; Field: Program type
 .db DEVELOPER_ID                   ; Type = Developer ID
#else
 .db $80, $12                       ; Field: Program type
 .db $01,$04                        ; Type = Shareware, TI-83+
#endif

 .db $80, $21                       ; Field: App ID
 .db 1                              ; Id = 1
 .db $80, $31                       ; Field: App Build
 .db 1                              ; Build = 1
 .db $80, $48                       ; Field: App Name
 .db "DIRECTX1"                     ; Name -- must be 8 characters
 .db $80, $81                       ; Field: App Pages
 .db 1                              ; App Pages = 1
 .db $80, $90                       ; No default splash screen
 .db $03, $26, $09, $04, $04, $6f, $1b, $80             ; Field: Date stamp- 5/12/1999
 .db $02, $0d, $40, $a1, $6b, $99, $f6, $59, $bc, $67   ; Dummy encrypted TI date stamp signature
 .db $f5, $85, $9c, $09, $6c, $0f, $b4, $03, $9b, $c9
 .db $03, $32, $2c, $e0, $03, $20, $e3, $2c, $f4, $2d 
 .db $73, $b4, $27, $c4, $a0, $72, $54, $b9, $ea, $7c 
 .db $3b, $aa, $16, $f6, $77, $83, $7a, $ee, $1a, $d4 
 .db $42, $4c, $6b, $8b, $13, $1f, $bb, $93, $8b, $fc 
 .db $19, $1c, $3c, $ec, $4d, $e5, $75
 .db $80, $7f                       ; Field: Program Image length
 .db   0, 0, 0, 0                   ; Length = 0, N/A
 .db   0, 0, 0, 0                   ; Reserved
 .db   0, 0, 0, 0                   ; Reserved
 .db   0, 0, 0, 0                   ; Reserved
 .db   0, 0, 0                      ; Reserved
#ifndef SIGN_APP
 .db 0      ; add another pad byte if using only 2-byte keyfile
#endif

swap_mem     .equ _SaveSScreen; as macrodie doesnt use this memory area we could use it as a swap memory

;// THIS MEMORY AREA BELONGS TO THE rotate cube but can be used for others when cube is not being called

camera         .equ 0
WorkCube       .equ 6;next at 12
intermediate   .equ 12;next at 14
distortion     .equ 14;distortion 
signed_flag    .equ 15
p              .equ 16;i will spare one here because it seems that something is overwrinting this var
CUBE           .equ 18;there exist 24 coords and the coords are only 1 byte so 17+24=41
angle_x        .equ 42
dist           .equ 43
SP_CUBE        .equ 44;there exist 8 points and 16 coords so 43 + 16 = 59
store_coords   .equ 60;one byte
angle_y        .equ 61
angle_z        .equ 62
angle_x_flag   .equ 63
angle_y_flag   .equ 64
angle_z_flag   .equ 65
animated_mode  .equ 66;detect to see if it is in animation mode
start_of_address_field .equ 67
address_1      .equ 67;address for the first face texture
address_2_1    .equ 68
address_2      .equ 69
address_2_2    .equ 70
address_3      .equ 71
address_2_3    .equ 72
address_4      .equ 73
address_2_4    .equ 74
address_5      .equ 75
address_2_5    .equ 76
address_6      .equ 77
address_2_6    .equ 78

;coordenadas do ecra (0,0,0) center of screen
;                     | | |
;                     x y z
;isto tem de ser convertido para coordenadas reais
;logo  quando X=0 =>x=48
;             Y=0 =>y=32
;             Z=0 =>z=0
;o Z nao tem de ser convertido pois o ecra e 2D
;o z apenas tem a distancia a que o cubo esta de nos 
;Um cubo tem 6 faces o que implica 8 vertices
;logo para obter um cubo temos de saber as coordenadas pela a seguinte ordem
; 
;                     
;                   1  _ _ _ _ 2                
;                  /|        /|               
;                   |         |
;             3 /_ _|_ _ _/4  |
;              |    |     |   |
;              |    |     |   | 
;              |   5|_ _ _|_ _|6 
;              |   /      |  /
;              |          |
;              |/_ _ _ _ _|/
;              7          8
;ScrPoint.x = Screen_center.x + (Constant * Vertex.x) / Vertex.z ;
;ScrPoint.y = Screen_center.y - (Constant * Vertex.y) / Vertex.z ;
;this in C thanks to the author of SolidWorks




;  Main app control begins here
StartApp:
    jp   error_ti_os;tios cannot execute this app
    jp   tell_macrodie_that_this_is_an_program;not driver and not library not macrodie
    jp   execute_background_task;this aplication executes a clock in background 
    RET \ RET \ RET ;\\ MirageOs execution prevent
    jp   initialyze_library;if you initialize the library you will have to hold the IX ram pointer and use only the routines for
                           ; working with unitialized library
    jp   dinitialize_library;this causes the library to stop and dealocate is ram area
    ;// this part refers to the cube library
    jp   create_wireframe_cube;\\ rotates a cube, if it is wireframe or not doesn't matter initialize create cube and denitialize
    jp   u_create_wireframe_cube;\\ creates a cube in unitialized library mode and rotate it by any angle
    ;// this part refers to the piramid library 
    jp   create_wireframe_piramid
    jp   u_create_wireframe_piramid
    ;// this part refers to an rectangular box library
    jp   create_wireframe_box
    jp   u_create_wireframe_box
    ;// well plans for working with solid are being maded
    
    
    

;// warning this library will be mutch more faster if you initialize and use the functions that require unitialized library
;// I will try to create a faster ILine  routine
            
    
    
    
    
    
    
    
    
    
;// this is opengl librarie created by rayden
;// it can create cubes only and animate them 
;// in the future it will create more than that
initialyze_library:
    ;the library uses 847 bytes but that can be changed if you include a new object into the library
    call  allocate_ram
    CP    1 \ jr   z,ram_alloc_error
    RET;return with no error
    
dinitialize_library:
    call  deallocate_ram
    CP    1 \ jr   z,ram_dealloc_error
    RET;go back
            
    
    
       
   

    
    
    
allocate_ram:;// this routine is used for allocate some temp ram    
    LD   HL,aplication_name
    call copy_to_OP1;OP1 active
    LD   HL,847;847B of free mem to reserve and IX can range from -128 to +128 for more we have to calculate the offset
    xor  a
    invoke( allocate_ram_area );allocate some ram for the system
    RET
    
deallocate_ram:;//this routine is used for dealocating the ram before allocated    
    LD   HL,aplication_name
    call copy_to_OP1;OP1 active
    xor  a
    invoke( deallocate_ram_area )
    RET
    
ram_dealloc_error:
    LD   A,1
    LD   B,4;deallocation error
    RET 
    
    
    
    
    
    
    
    
ram_alloc_error:    
   LD  A,1
   LD  B,A
   RET
   
   
  
   
   
   
   
   

   
   
   

       
Div_HL_D1:            ; HL = HL  D, A = remainder
    XOR    A         ; Clear upper eight bits of AHL
    LD     B, 16      ; Sixteen bits in dividend
_loop1:
    ADD    HL, HL     ; Do a SLA HL
    RLA              ; This moves the upper bits of the dividend into A
    JR     C, _overflow
    CP     D         ; Check if we can subtract the divisor
    JR     C, _skip1   ; Carry means D > A
_overflow:
    SUB    D         ; Do subtraction for real this time
    INC    L         ; Set bit 0 of quotient
_skip1:
    DJNZ   _loop1
    RET
                                          
                       






DE_Times_A:          ; HL = DE  A
    LD     HL, 0      ; Use HL to store the product
    LD     B, 8       ; Eight bits to check
_loop:
    RRCA             ; Check least-significant bit of accumulator
    JR     NC, _skip  ; If zero, skip addition
    ADD    HL, DE
_skip:
    SLA    E         ; Shift DE one bit left
    RL     D
    DJNZ   _loop
    RET



       
              

                     
                                            
                            
                                          
erase_my_ram_space              
    ;the place of ram comes in IX
    PUSH HL
    PUSH DE
    PUSH IX
    PUSH BC
    PUSH IX \ POP DE
    LD  H,D \ LD  L,E
    INC DE
    LD  (HL),0;erase that byte
    LD  BC,99
    LDIR;100 bytes to erase
    POP  BC
    POP  IX
    POP  DE
    POP  HL
    RET
   
   
   
   
 





execute_background_task:
;blink cursor and update it to screen
    RET;para ja volta apenas para traz pois ainda nao tem nada defenido    













putsprite_function:
    PUSH  HL
    PUSH  BC
    PUSH  IX
    POP   HL 
    LD    DE,swap_mem
    LD    BC,16;16 bytes a copiar
    LDIR
    POP   BC
    POP   HL
    LD    IX,swap_mem
    xor  a;macrodie page
    invoke( putsprite ); poe me o sprite no ecra
    RET
copy_to_OP1:
    LD   DE,_OP1
    LD   BC,8
    LDIR
    RET








#include "CUBE.asm"
#include "BOX.asm";this refers to the creation of an BOX
#include "PIRAMID.asm";and this refers to the creation of an PIRAMID 
#include "trigonometry.asm";this one provide the basic routines for the creation of each object



















        
    






tell_macrodie_that_this_is_an_program:
    LD    A,2;//this is a librarie
    RET;return to whatever




error_ti_os:
    DI
    bcall _GRBUFCLR
    EI
    LD  HL,error_ti_os1
    LD  DE,buffer
    LD  BC,error_end - error_ti_os1
    LDIR
    LD  HL,buffer
    LD  DE,$0000
    LD  (_penCol),DE
    bcall _VPutS
    bcall _GetKey
    
    bjump _JForceCmdNoChar


    
    
error_ram_alloc: .db "Error allocating ram please unload some aplications in order to run this one!!! Please press Enter Key!!!",0   
error_ti_os1: .db "Error Ti-OS cann't execute this aplication",0
error_end:
aplication_name: .db "DIRECTX1",0;the directx memory area

















.end
.end
